/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.readGraph.adapt;

import org.simantics.db.AsyncReadGraph;
import org.simantics.db.ReadGraph;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.ResourceAsyncRead;
import org.simantics.db.exception.AdaptionException;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.procedure.AsyncProcedure;
import org.simantics.db.testing.base.WriteReadTest;
import org.simantics.db.tests.api.readGraph.common.Ontologies;

/**
 * Checks that {@link ReadGraph#adapt(org.simantics.db.Resource, Class)},
 * {@link ReadGraph#adaptUnique(org.simantics.db.Resource, Class)},
 * <code>AsyncReadGraph.forAdapted</code> fail as expected.
 * 
 * <p>
 * Regression test for https://www.simulationsite.net/trac/simantics/ticket/617
 * 
 * @author Tuukka Lehtonen
 */
public class AdaptTest4 extends WriteReadTest {

	@Override
	protected void write(WriteGraph graph) throws DatabaseException {
		written = Ontologies.relatedTypelessResource(L0, graph);
	}

	@Override
	protected void read(ReadGraph graph) throws DatabaseException {
		try {

			String s = graph
					.syncRequest(new ResourceAsyncRead<String>(written) {
						@Override
						public void perform(AsyncReadGraph graph,
								AsyncProcedure<String> procedure) {
							graph.forAdapted(resource, String.class, procedure);
						}
					});
			assertTrue("adapt must never return null", s != null);
		} catch (AdaptionException e) {
			return;
		} catch (Exception e) {
			fail("Wrong exception: " + e.getClass().getCanonicalName());
		}
		fail("Should throw");
	}
}
