/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.request.exception;

import org.junit.Test;
import org.simantics.db.AsyncReadGraph;
import org.simantics.db.Session;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.procedure.AsyncProcedure;
import org.simantics.db.request.AsyncRead;
import org.simantics.db.testing.base.ExistingDatabaseTest;

public class SessionSyncAsyncReadThrowsRuntimeException extends ExistingDatabaseTest {
	private RuntimeException exception; 
	
	@Test
	public void testWriteException() throws Exception {
		
		Session session = getSession();
		
		try {

			session.syncRequest(new AsyncRead<Object>() {

                @Override
                public void perform(AsyncReadGraph graph, AsyncProcedure<Object> procedure) {
                    exception = new IllegalArgumentException();
                    throw exception;
                }

                @Override
    		    public int threadHash() {
    		    	return hashCode();
    		    }

                @Override
                public int getFlags() {
                    return 0;
                }

			});
		
		} catch (Throwable e) {

		    if(!(e instanceof DatabaseException)) {
                fail("Request did not throw DatabaseException but " + e);
                return;
		    }

		    DatabaseException ex = (DatabaseException)e;
		    
			if(!exception.equals(ex.getCause())) {
				fail("Request did not report the correct cause but " + e);
	            return;
			}

			if (DEBUG)
				e.printStackTrace();
			
			return;
			
		}
		
		fail("Request did not throw an exception.");
		
	}

}
