/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.request.exception;

import org.junit.Test;
import org.simantics.db.ReadGraph;
import org.simantics.db.Session;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.procedure.Procedure;
import org.simantics.db.request.Read;
import org.simantics.db.testing.base.ExistingDatabaseTest;

public class SessionSyncReadWithProcedureThrowsDatabaseException extends ExistingDatabaseTest {
	
    private String failure = null;
	private Throwable exception; 
	
	@Test
	public void test() throws Exception {
		
		Session session = getSession();
		
		try {
		
    		session.syncRequest(new Read<Object>() {
    
    			@Override
    			public Object perform(ReadGraph graph) throws DatabaseException {
    			    
    			    DatabaseException e = new DatabaseException("Test");
    			    exception = e;
    			    throw e;
    
    			}
    			
    		}, new Procedure<Object>() {

                @Override
                public void exception(Throwable t) {
                    
                    if(t != exception) failure = "syncRequest did not throw the original exception";
                    
        		    if (DEBUG)
        		    	t.printStackTrace();
                   
                }

                @Override
                public void execute(Object result) {
                    
                    failure = "Execute was called with " + result;
                    
                }
    		    
    		});
    		
		} catch(DatabaseException e) {
		    
		    if(e != exception) fail("syncRequest did not throw the original exception");
		    if(failure != null) fail(failure);
		    
		    if (DEBUG)
		    	e.printStackTrace(); 
		    
		    return;
		    
		} catch(Throwable t) {
		    
	        fail("syncRequest threw unexpected throwable " + t);

		}
		
		fail("syncRequest did not throw");
		
	}

}
