/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.write.request;

import org.junit.Test;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.testing.base.ExistingDatabaseTest;

/**
 * @author Tuukka Lehtonen
 *
 */
public class SessionWriteErrorTest extends ExistingDatabaseTest {
	
	private Error exception; 
	
	@Test
	public void testWriteException() throws Exception {
		
		Session session = getSession();
		
		try {

			session.syncRequest(new WriteRequest() {

				@Override
				public void perform(WriteGraph graph) throws DatabaseException {

					exception = new Error();
					throw exception;

				}

			});
		
		} catch (Throwable e) {

			if(e.getCause() != exception) {
				fail("Write transaction threw an unknown exception " + e);
			}

			// Check that the session bookkeeping is still intact for another write transaction.
			session.syncRequest(new WriteRequest() {
				@Override
				public void perform(WriteGraph graph) throws DatabaseException {
				}
			});

			return;
			
		}
		
		fail("Write transaction did not throw an exception.");
		
	}

}
