/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.write.request;

import org.junit.Test;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.testing.annotation.Fails;
import org.simantics.db.testing.base.ExistingDatabaseTest;

public class WriteGraphWriteExceptionTest extends ExistingDatabaseTest {
	
	private DatabaseException exception; 
	
	@Test
	@Fails
	public void testWriteException() throws Exception {
		
		Session session = getSession();
		
		try {
		
			session.syncRequest(new WriteRequest() {
	
				@Override
				public void perform(WriteGraph graph) throws DatabaseException {
	
					try {
	
						graph.syncRequest(new WriteRequest() {
	
							@Override
							public void perform(WriteGraph graph) throws DatabaseException {
	
								exception = new DatabaseException();
								throw exception;
	
							}
	
						});
	
					} catch (Throwable e) {
	
						if(!e.equals(exception)) {
							fail("Write transaction threw an unknown exception.");
						}
	
						if(e instanceof DatabaseException) throw (DatabaseException)e;
	
					}
					
					fail("Write request (Graph) did not throw the exception.");
	
				}
	
			});
			
			fail("Write transaction (Session) did not throw the exception.");
		
		} catch (DatabaseException e) {
			
			if(e.equals(exception)) {

				// Check that the session bookkeeping is still intact for another write transaction.
				session.syncRequest(new WriteRequest() {
					@Override
					public void perform(WriteGraph graph) throws DatabaseException {
					}
				});

				return;
			}
			
			fail("Write transaction did not throw the original exception but instead threw " + e);
			
		}

	}

}
