/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.performance.read;

import java.util.Set;

import org.simantics.databoard.Bindings;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.common.request.ResourceRead;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.service.CollectionSupport;
import org.simantics.layer0.Layer0;
import org.simantics.utils.datastructures.Pair;

public class TypeSetAndString extends ResourceRead<Pair<Set<Resource>, String>> {

    public TypeSetAndString(Resource type) {
        super(type);
    }

    @Override
    public Pair<Set<Resource>, String> perform(ReadGraph graph) throws DatabaseException {
    	
    	Layer0 L0 = Layer0.getInstance(graph);
    	CollectionSupport cs = graph.getService(CollectionSupport.class);
    	Set<Resource> types = cs.createSet();
    	
    	types.add(resource);
    	types.addAll(graph.getSupertypes(resource));
    	
        final StringBuilder b = new StringBuilder();
        boolean first = true;
        for (Resource type : types) {
        	String name = graph.getPossibleRelatedValue(type, L0.HasName, Bindings.STRING);
            if (!first) b.append(" ");
            first = false;
        }
        
        return Pair.make(types, b.toString());
        
    }

}