package org.simantics.db.tests.regression.bugs;

import java.util.UUID;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.WriteOnlyGraph;
import org.simantics.db.common.request.ReadRequest;
import org.simantics.db.common.request.WriteOnlyRequest;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.service.ClusterControl;
import org.simantics.db.testing.annotation.Fails;
import org.simantics.db.testing.cases.FreshDatabaseTest;
import org.simantics.db.testing.common.Terminator;
import org.simantics.db.testing.common.TestBase;
import org.simantics.db.testing.impl.Configuration;
import org.simantics.layer0.Layer0;

public class Issue3199Test1 extends FreshDatabaseTest {
    static int RESOURCE_COUNT = Configuration.get().i3199ResourceCount;
    static boolean DEBUG = false;
    Session session;
    Resource testRoot;
    ClusterControl clusterControl;
    
    private Terminator terminator;
    
    @Before
    @Override
    public void setUp() throws Exception {
    	super.setUp();
    	terminator = new Terminator(3);
    }

    @After
    @Override
    public void tearDown() throws Exception {
    	terminator.disarm();
    	super.tearDown();
    }
    
    @Test
    @Fails
    public void testSimanticsIssue3199()
    throws DatabaseException {
        session = getSession();
        session.syncRequest(new Init());
        session.syncRequest(new Create());
        session.syncRequest(new Query());
    }
    class Init extends WriteRequest {
        @Override
        public void perform(WriteGraph g) throws DatabaseException {
            Layer0 l0 = Layer0.getInstance(g);
            Resource rl = g.getResource(TestBase.ROOT_LIBRARY_URI);
            testRoot = g.newResource();
            g.claim(testRoot, l0.InstanceOf, l0.Library);          
            g.claim(rl, l0.ConsistsOf, testRoot);
        }
    }
    class Create extends WriteOnlyRequest {
        @Override
        public void perform(WriteOnlyGraph g) throws DatabaseException {
            Layer0 l0 = Layer0.getInstance(getSession());
            Resource root = g.newResource();
            g.claim(testRoot, l0.ConsistsOf, l0.PartOf, root);
            for (int i = 0; i < RESOURCE_COUNT; i++) {
                Resource item = g.newResource();
                g.claim(item, l0.InstanceOf, null, l0.String);
                g.claim(item, l0.IsWeaklyRelatedTo, null, l0.Abstract);
                g.claim(item, l0.IsWeaklyRelatedTo, null, l0.Assertion);
                g.claim(item, l0.IsWeaklyRelatedTo, null, l0.Graph);
                g.claim(item, l0.IsWeaklyRelatedTo, null, l0.Entity);
                g.claim(item, l0.IsWeaklyRelatedTo, null, l0.Relation);
                g.claimValue(item, UUID.randomUUID().toString());
                g.claim(root, l0.ConsistsOf, l0.PartOf, item);
            }
        }
    }
    class Query extends ReadRequest {
        @Override
        public void run(ReadGraph g) throws DatabaseException {
            Layer0 l0 = Layer0.getInstance(g);
            for (Resource r : g.getObjects(testRoot, l0.ConsistsOf)) {
                if (DEBUG)
                    System.out.println("Resource " + r);
                for (Resource rr : g.getObjects(r, l0.ConsistsOf)) {
                    if (!g.isInstanceOf(rr, l0.String))
                        fail("Resource " + rr + " is not instance of String.");
                    if (!g.isInstanceOf(rr, l0.Entity))
                        fail("Resource " + rr + " is not instance of Entity.");
                }
            }
        }
    }
}
