package org.simantics.db.tests.regression.bugs;

import java.util.UUID;

import org.junit.Test;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.WriteOnlyGraph;
import org.simantics.db.common.request.ReadRequest;
import org.simantics.db.common.request.WriteOnlyRequest;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.layer0.util.SessionGarbageCollection;
import org.simantics.db.service.ClusterControl;
import org.simantics.db.service.ManagementSupport;
import org.simantics.db.service.XSupport;
import org.simantics.db.testing.cases.FreshDatabaseTest;
import org.simantics.db.testing.common.TestBase;
import org.simantics.db.testing.impl.Configuration;
import org.simantics.layer0.Layer0;

public class Issue3199Test2 extends FreshDatabaseTest {
    static final int RESOURCE_COUNT = Configuration.get().i3199ResourceCount; // 10 * 1000 * 1000;
    static final int CONSIST_COUNT = 500; // How many resources in the same parent.
    static final int COMMIT_COUNT = 50 * 1000; // How many resources in the same commit.
    static final boolean DEBUG = false;
    static final boolean VERBOSE = false;
    Session session;
    Resource testRoot;
    ClusterControl clusterControl;

    @Test
    public void testSimanticsIssue3199()
            throws DatabaseException {
        session = getSession();
        session.syncRequest(new Init());
        long start = System.nanoTime();
        session.syncRequest(new Create());
        long duration = System.nanoTime() - start;
        if (DEBUG)
            System.out.println("Write done in " + 1e-9 * duration + " seconds.");
        start = System.nanoTime();
        session.syncRequest(new Query());
        long total = duration;
        if (DEBUG) {
            duration = System.nanoTime() - start;
            total += duration;
            System.out.println("Read done in " + 1e-9 * duration + " seconds.");
            System.out.println("Both done in " + 1e-9 * total + " seconds.");
        }
    }

    class Init extends WriteRequest {
        @Override
        public void perform(WriteGraph g) throws DatabaseException {
            Layer0 l0 = Layer0.getInstance(g);
            Resource rl = g.getResource(TestBase.ROOT_LIBRARY_URI);
            testRoot = g.newResource();
            g.claim(testRoot, l0.InstanceOf, l0.Library);
            g.claim(rl, l0.ConsistsOf, testRoot);
        }
    }

    class Create extends WriteOnlyRequest {
        @SuppressWarnings("unused")
        @Override
        public void perform(WriteOnlyGraph g) throws DatabaseException {
            Layer0 l0 = Layer0.getInstance(getSession());
            Resource root = g.newResource();
            g.claim(testRoot, l0.ConsistsOf, l0.PartOf, root);
            ManagementSupport ms = g.getService(ManagementSupport.class);
            XSupport xs = g.getService(XSupport.class);
            for (int i = 0, j = 1, k = 1; i < RESOURCE_COUNT; i++, j++, k++) {
                Resource item = g.newResource();
                g.claim(item, l0.InstanceOf, null, l0.String);
                g.claim(item, l0.IsWeaklyRelatedTo, null, l0.Abstract);
                g.claim(item, l0.IsWeaklyRelatedTo, null, l0.Assertion);
                g.claim(item, l0.IsWeaklyRelatedTo, null, l0.Graph);
                g.claim(item, l0.IsWeaklyRelatedTo, null, l0.Entity);
                g.claim(item, l0.IsWeaklyRelatedTo, null, l0.Relation);
                g.claimValue(item, UUID.randomUUID().toString());
                if (j == CONSIST_COUNT) {
                    j = 0;
                    root = g.newResource();
                    g.claim(testRoot, l0.ConsistsOf, l0.PartOf, root);
                }
                assert (null != root);
                g.claim(root, l0.ConsistsOf, l0.PartOf, item);
                if (k == COMMIT_COUNT) {
                    if (DEBUG && VERBOSE)
                        System.out.println("resource count=" + (i + 1));
                    k = 0;
                    long beg = ms.getHeadRevisionId();
                    xs.commitAndContinue(g, this);
                    long end = ms.getHeadRevisionId();
                    assertTrue(beg + 1 == end);
                }
            }
        }
    }

    class Query extends ReadRequest {
        @SuppressWarnings("unused")
        @Override
        public void run(ReadGraph g) throws DatabaseException {
            Layer0 l0 = Layer0.getInstance(g);
            long count = 0;
//            ClusterControl cc = g.getService(ClusterControl.class);
            for (Resource r : g.getObjects(testRoot, l0.ConsistsOf)) {
                for (Resource rr : g.getObjects(r, l0.ConsistsOf)) {
                    if (!g.isInstanceOf(rr, l0.String))
                        fail("Resource " + rr + " is not instance of String.");
                    if (!g.isInstanceOf(rr, l0.Entity))
                        fail("Resource " + rr + " is not instance of Entity.");
                    ++count;
                    if (DEBUG && VERBOSE && count % 100000 == 0)
                        System.out.println("resource count=" + count);
                }
                // cc.gc(g);
                SessionGarbageCollection.gc(g, Integer.MAX_VALUE, 0);
            }
            assertTrue(count == RESOURCE_COUNT);
        }
    }
}
